@@ -150,6 +150,65 @@ repository.
 History
 -------
 
+hgsvn 0.1.8
+-----------
+
+Improvements:
+
+* Convert a local SVN checkout into a hgsvn controlled Mercurial
+  repository with ``hgimportsvn --local-only``. No network access is
+  needed when setting this flag (aka airplane mode). The Mercurial
+  history then starts with the current revision of the SVN
+  checkout. Patch by Matt Fowles.
+
+* Commit messages when pushing back to SVN can be edited before
+  committing using the -e/--edit command line flag. Issue #29, patch
+  by eliterr.
+
+* It's now possible to use ``hgsvn`` with the mq extensions. Both
+  ``hgpushsvn`` and ``hgpullsvn`` abort with an error message if mq
+  patches are applied when running those commands. Issue #43, patch by
+  sterin.
+
+Bug fixes:
+
+* Removal of temporary file when pushing new revisions to SVN on
+  Windows fixed. Issue 8 reported by Daniel Dabrowski.
+
+* Changed 'hg log' command line arguments that interfered with default
+  options in .hgrc file. Issue 12 reported by Simon Percivall, initial
+  patch by Joel Rosdahl and issue 16 reported by Wladimir Palant.
+
+* ``hgpushsvn`` is now compatible with Python 2.4.
+
+* If a SVN changeset contains empty changeset comments, ``hgpullsvn``
+  has failed. Empty changeset messages are now handled correctly.
+
+* SVN repositories at revision 0 couldn't be imported. Now it's
+  possible to import empty SVN repositories with ``hgimportsvn``.
+  Issue #13 reported by tiktuk.
+
+* ``hgpushsvn`` failed when a non-empty directory was removed from
+  version control. Issue #15 reported by Keith Yang.
+
+* Proper encoding of commit messages on Windows systems. Issue #19
+  reported and patch by Chunlin Yao.
+
+* Change svn:executable property on mode changes when pushing back to
+  SVN repository. Issue #24 reported by sterin.
+
+* Improved parsing of SVN messages. Issues #27, #14 patch contributed
+  by x63.
+
+* ``hgpushsvn`` whiped uncommitted changes in working directory. Issue
+  #32 reported by foxcub.
+
+* ``hgpullsvn`` in dry-run-mode displayed latest fetched revision
+  too. Issue #35 reported by Dmitriy Morozov.
+
+* New sub-directories are now created properly. Issue #46 reported by
+  Matt Fowles.
+
 hgsvn 0.1.7
 -----------
 
@@ -358,7 +417,7 @@ __all__ = []
 
 __author__ = 'Antoine Pitrou, Andi Albrecht'
 __license__ = 'GNU General Public License (version 3 or later)'
-__versioninfo__ = (0, 1, 7)
+__versioninfo__ = (0, 1, 8)
 
 base_version = '.'.join(map(str, __versioninfo__))
 full_version = base_version
