@@ -156,9 +156,10 @@ def run_command(cmd, args=None, bulk_args=None, encoding=None, fail_if_stderr=Fa
     Run a command without using the shell.
     """
     args = args or []
+    bulk_args = bulk_args or []
     def _transform_arg(a):
         if isinstance(a, unicode):
-            a = a.encode(encoding or locale_encoding)
+            a = a.encode(encoding or locale_encoding or 'UTF-8')
         elif not isinstance(a, str):
             a = str(a)
         return a
@@ -214,14 +215,18 @@ def run_shell_command(cmd, args=None, bulk_args=None, encoding=None):
         i = stop
     return out
 
-def run_hg(args=None, bulk_args=None):
+def run_hg(args=None, bulk_args=None, output_is_locale_encoding=False):
     """
     Run a Mercurial command, returns the (unicode) output.
     """
     default_args = ["--encoding", "utf-8"]
     output = run_command("hg", args=default_args + (args or []),
-        bulk_args=bulk_args, encoding="utf-8")
-    return output.decode('utf-8')
+        bulk_args=bulk_args)
+    if output_is_locale_encoding:
+        enc = locale_encoding
+    else:
+        enc = 'utf-8'
+    return output.decode(enc)
 
 def run_svn(args=None, bulk_args=None, fail_if_stderr=False):
     """
@@ -254,18 +259,18 @@ def hg_commit_from_svn_log_entry(entry, files=None):
     message = (hg_commit_prefix % entry['revision']) + entry['message'].lstrip()
     commit_file = None
     try:
-        # Very long commit messages must not be passed on the command line,
-        # or command invokation can fail on some platforms.
-        if len(message) > 16000:
-            commit_file = os.path.join(hgsvn_private_dir,
-                "commit-%r.txt" % entry['revision'])
-            f = open(commit_file, "wb")
-            f.write(message)
-            f.close()
-            msg_options = ["-l", commit_file]
-        else:
-            msg_options = ["-m", message]
-        options = ["ci"] + msg_options + ["-u", entry['author'], "-d", hg_date]
+        commit_file = os.path.join(hgsvn_private_dir,
+            "commit-%r.txt" % entry['revision'])
+        f = open(commit_file, "wb")
+        f.write(message.encode('utf-8'))
+        f.close()
+        msg_options = ["-l", commit_file]
+        # the -m will now work on windows with utf-8 encoding argument
+        # the CreateProcess win32api convert bytes to uicode by locale codepage
+        # msg.encode('utf-8').decode('cp932').encode('cp932').decode('utf-8')
+        options = ["ci"] + msg_options + ["-d", hg_date]
+        if entry['author']:
+            options.extend(["-u", entry['author']])
         if files:
             run_hg(options, files)
         else:
@@ -334,7 +339,7 @@ def change_to_rootdir():
 def hg_is_clean(current_branch):
     """Returns False if the local Mercurial repository has
        uncommitted changes."""
-    if run_hg(['st', '-mard']).strip():
+    if run_hg(['st', '-mard'], output_is_locale_encoding=True).strip():
         ui.status(("\nThe Mercurial working copy contains pending changes "
                    "in branch '%s'!\n"
                    "Please either commit or discard them before running "
@@ -367,3 +372,18 @@ def hg_switch_branch(current_branch, new_branch):
         run_hg(['up', '-C', new_branch])
     run_hg(["branch", new_branch])
     return True
+
+
+def check_for_applied_patches():
+    """Check for applied mq patches.
+
+    Returns ``True`` if any mq patches are applied, otherwise ``False``.
+    """
+    try:
+        out = run_hg(["qapplied"])
+        patches = [s.strip() for s in out.splitlines()]
+        if len(patches) > 0:
+            return True
+    except ExternalCommandFailed, err:
+        pass
+    return False
