@@ -30,7 +30,8 @@ from optparse import OptionParser
 
 
 def main():
-    usage = "usage: %prog [-r SVN rev] [-p SVN peg rev] <SVN URL> [local checkout]"
+    usage = """%prog [-r SVN rev] [-p SVN peg rev] <SVN URL> [local checkout]
+        %prog --local-only <local checkout>"""
     parser = OptionParser(usage)
     parser.add_option("-r", "--svn-rev", type="int", dest="svn_rev",
         help="SVN revision to checkout from")
@@ -38,39 +39,71 @@ def main():
         help="SVN peg revision to locate checkout URL")
     parser.add_option("--no-hgignore", dest="hgignore", action="store_false",
         default=True, help="Don't autogenerate .hgignore")
+    parser.add_option("--local-only", action="store_true", default=False,
+        help="Don't use the server, just build from a local checkout")
     (options, args) = run_parser(parser, __doc__)
     if not 1 <= len(args) <= 2:
         display_parser_error(parser, "incorrect number of arguments")
 
-    svn_url = args.pop(0).rstrip("/")
+    target_svn_url = args.pop(0).rstrip("/")
     local_repo = args and args.pop(0) or None
     if options.svn_peg:
-       svn_url += "@" + str(options.svn_peg)
+       target_svn_url += "@" + str(options.svn_peg)
+
+    if options.local_only:
+        if options.svn_peg:
+            display_parser_error(parser,
+                    "--local-only and --svn-peg are mutually exclusive")
+        if options.svn_rev:
+            display_parser_error(parser,
+                    "--local-only and --svn-rev are mutually exclusive")
+        if local_repo:
+            display_parser_error(parser,
+                    "--local-only does not accept a target directory")
+
+
 
     # Get SVN info
-    svn_info = get_svn_info(svn_url, options.svn_rev)
+    svn_info = get_svn_info(target_svn_url, options.svn_rev)
     # e.g. u'svn://svn.twistedmatrix.com/svn/Twisted'
     repos_url = svn_info['repos_url']
     # e.g. u'svn://svn.twistedmatrix.com/svn/Twisted/branches/xmpp-subprotocols-2178-2'
-    svn_url = svn_info['url']
-    assert svn_url.startswith(repos_url)
+    actual_svn_url = svn_info['url']
+    assert actual_svn_url.startswith(repos_url)
+
+    # if we are allowed to go to the server, lets use it
+    if options.local_only:
+        target_svn_url = os.path.abspath(target_svn_url)
+        local_repo = target_svn_url
+
     # e.g. u'/branches/xmpp-subprotocols-2178-2'
-    svn_path = svn_url[len(repos_url):]
+    svn_path = actual_svn_url[len(repos_url):]
     # e.g. 'xmpp-subprotocols-2178-2'
-    svn_branch = svn_url.split("/")[-1]
+    svn_branch = actual_svn_url.split("/")[-1]
     svn_greatest_rev = svn_info['last_changed_rev']
     if options.svn_peg:
-       svn_url += "@" + str(options.svn_peg)
+       actual_svn_url += "@" + str(options.svn_peg)
 
     if not local_repo:
         local_repo = svn_branch
     if os.path.exists(local_repo):
         if not os.path.isdir(local_repo):
             raise ValueError("%s is not a directory" % local_repo)
-        if os.path.isdir(os.path.join(local_repo, '.svn')):
+        is_svn_dir = os.path.isdir(os.path.join(local_repo, '.svn'))
+        if is_svn_dir and not options.local_only:
             nlocal_repo = os.path.abspath(local_repo)
             print "%s is already a SVN checkout." % nlocal_repo
             sys.exit(1)
+        elif not is_svn_dir and options.local_only:
+            nlocal_repo = os.path.abspath(local_repo)
+            print "%s must be a SVN checkout for --local-only" % nlocal_repo
+            sys.exit(1)
+        elif options.local_only and is_svn_dir:
+            status = get_svn_status(local_repo, quiet=True)
+            if status:
+                print ("There are uncommitted changes. Either commit them "
+                       "or put them aside before running hgimportsvn.")
+                sys.exit(1)
     else:
         os.mkdir(local_repo)
     fixup_hgsvn_dir(local_repo)
@@ -79,12 +112,19 @@ def main():
     # Get log entry for the SVN revision we will check out
     svn_copyfrom_path = None
     svn_copyfrom_revision = None
-    if options.svn_rev:
+    if options.local_only or svn_greatest_rev == 0:
+        # fake up a log message for the initial commit
+        svn_start_log = {}
+        svn_start_log['message'] = 'initial import by hgimportsvn'
+        svn_start_log['revision'] = svn_info['last_changed_rev']
+        svn_start_log['author'] = svn_info.get('last_changed_author')
+        svn_start_log['date'] = svn_info['last_changed_date']
+    elif options.svn_rev:
         # If a specific rev was requested, get log entry just before or at rev
-        svn_start_log = get_last_svn_log_entry(svn_url, 1, options.svn_rev)
+        svn_start_log = get_last_svn_log_entry(actual_svn_url, 1, options.svn_rev)
     else:
         # Otherwise, get log entry of branch creation
-        svn_start_log = get_first_svn_log_entry(svn_url, 1, svn_greatest_rev)
+        svn_start_log = get_first_svn_log_entry(actual_svn_url, 1, svn_greatest_rev)
         for p in svn_start_log['changed_paths']:
             if p['path'] == svn_path:
                 svn_copyfrom_path = p['copyfrom_path']
@@ -131,10 +171,80 @@ def main():
                     run_hg(["up", "tip"])
     run_hg(["branch", svn_branch])
 
-    svn_ignores = filter(None, [s.strip()
-        for s in run_svn(['propget', 'svn:ignore', svn_url]).splitlines()])
+
+    # Stay on the same filesystem so as to have fast moves
+    if options.local_only:
+        checkout_dir = target_svn_url
+    else:
+        checkout_dir = tempfile.mkdtemp(dir=".")
+
+    try:
+        # Get SVN manifest and checkout
+        if not options.local_only:
+            svn_checkout(target_svn_url, checkout_dir, svn_rev)
+        svn_manifest = get_svn_versioned_files(checkout_dir)
+
+        svn_files = set(skip_dirs(svn_manifest, checkout_dir))
+        svn_dirs = sorted(set(svn_manifest) - svn_files)
+        svn_files = list(svn_files)
+
+        # in the local case the files here already
+        if not options.local_only:
+            # All directories must exist, including empty ones
+            # (both for hg and for moving .svn dirs later)
+            for d in svn_dirs:
+                if not os.path.isdir(d):
+                    if os.path.exists(d):
+                        os.remove(d)
+                    os.mkdir(d)
+            # Replace checked out files
+            for f in svn_files:
+                if os.path.exists(f):
+                    os.remove(f)
+                os.rename(os.path.join(checkout_dir, f), f)
+
+        try:
+            # Add/remove new/old files
+            if svn_files:
+                run_hg(["addremove"] + hg_exclude_options, svn_files)
+            hg_commit_from_svn_log_entry(svn_start_log)
+            #hg_commit_from_svn_log_entry(svn_start_log, svn_files)
+        except:
+            run_hg(["revert", "--all"])
+            raise
+
+        # in the local case the files here already
+        if not options.local_only:
+            # Move SVN working copy here (don't forget base directory)
+            for d in chain([""], svn_dirs):
+                os.rename(os.path.join(checkout_dir, d, svn_private_dir), os.path.join(d, svn_private_dir))
+
+    finally:
+        # in the local case the checkout_dir is the target, so don't delete it
+        if not options.local_only:
+            rmtree(checkout_dir)
 
     if options.hgignore:
+        svn_ignores = []
+        # we use '.' because that it the location of the hg repo that we are
+        # building and at this point we have already copied all of svn to the
+        # checkout (for both the local and non-local case)
+        for (path, dirs, files) in os.walk('.'):
+            if '.svn' in dirs:
+                dirs.remove('.svn')
+
+                # the [2:] strips off the initial './'
+                local_ignores = [os.path.join(path, s.strip())[2:]
+                    for s in run_svn(['propget', 'svn:ignore', path]).splitlines()
+                    if bool(s.strip())
+                ]
+                svn_ignores += local_ignores
+            else:
+                # if we are not inside an svn world
+                # we can just bail
+                del dirs[:]
+
+
         # Generate .hgignore file to ignore .svn and .hgsvn directories
         f = open(".hgignore", "a")
         try:
@@ -148,52 +258,8 @@ def main():
         finally:
             f.close()
 
-    # Stay on the same filesystem so as to have fast moves
-    checkout_dir = tempfile.mkdtemp(dir=".")
-
-    try:
-        # Get SVN manifest and checkout
-        svn_checkout(svn_url, checkout_dir, svn_rev)
-        svn_manifest = get_svn_versioned_files(checkout_dir)
-
-        svn_files = set(skip_dirs(svn_manifest, checkout_dir))
-        svn_dirs = sorted(set(svn_manifest) - svn_files)
-        svn_files = list(svn_files)
-
-        # All directories must exist, including empty ones
-        # (both for hg and for moving .svn dirs later)
-        for d in svn_dirs:
-            if not os.path.isdir(d):
-                if os.path.exists(d):
-                    os.remove(d)
-                os.mkdir(d)
-        # Replace checked out files
-        for f in svn_files:
-            if os.path.exists(f):
-                os.remove(f)
-            os.rename(os.path.join(checkout_dir, f), f)
-
-        try:
-            # Add/remove new/old files
-            if svn_files:
-                run_hg(["addremove"] + hg_exclude_options, svn_files)
-            hg_commit_from_svn_log_entry(svn_start_log)
-            #hg_commit_from_svn_log_entry(svn_start_log, svn_files)
-        except:
-            run_hg(["revert", "--all"])
-            raise
-
-        # Move SVN working copy here (don't forget base directory)
-        for d in chain([""], svn_dirs):
-            os.rename(os.path.join(checkout_dir, d, svn_private_dir), os.path.join(d, svn_private_dir))
-
-    finally:
-        rmtree(checkout_dir)
-        pass
-
     print "Finished! You can now pull all SVN history with 'hgpullsvn'."
 
 
 if __name__ == "__main__":
     main()
-
