@@ -9,6 +9,7 @@ from hgsvn.common import (
     run_hg, run_svn,
     hg_commit_from_svn_log_entry, hg_exclude_options,
     get_svn_rev_from_hg, hg_switch_branch, hg_is_clean,
+    check_for_applied_patches,
 )
 from hgsvn.svnclient import (
     get_svn_info, get_svn_versioned_files, iter_svn_log_entries, run_svn_log,
@@ -97,7 +98,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                 else:
                     # Extract actual copied files (hg doesn't track dirs
                     # and will refuse "hg copy -A" with dirs)
-                    r = run_hg(["st", "-nc"], [old_p])
+                    r = run_hg(["st", "-nc"], [old_p], output_is_locale_encoding=True)
                     for old_f in r.splitlines():
                         f = p + old_f[len(old_p):]
                         copied_paths.append((old_f, f))
@@ -154,7 +155,7 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                 except ExternalCommandFailed:
                     # Maybe the "old" path is obsolete, i.e. it comes from an
                     # old SVN revision and was later removed.
-                    s = run_hg(['st', '-nd'], [old])
+                    s = run_hg(['st', '-nd'], [old], output_is_locale_encoding=True)
                     if s.strip():
                         # The old path is known by hg, something else happened.
                         raise
@@ -182,6 +183,10 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
 
 
 def real_main(options, args):
+    if check_for_applied_patches():
+        print ("There are applied mq patches. Put them aside before "
+               "running hgpsuhsvn.")
+        return 1
     svn_wc = "."
     svn_peg_wc = svn_wc
     if options.svn_peg:
@@ -195,6 +200,8 @@ def real_main(options, args):
     repos_url = svn_info['repos_url']
     # e.g. u'svn://svn.twistedmatrix.com/svn/Twisted/branches/xmpp-subprotocols-2178-2'
     wc_url = svn_info['url']
+    if options.svn_peg:
+        svn_peg_wc = wc_url + "@" + str(options.svn_peg)
     assert wc_url.startswith(repos_url)
     # e.g. u'/branches/xmpp-subprotocols-2178-2'
     wc_base = wc_url[len(repos_url):]
@@ -214,10 +221,13 @@ def real_main(options, args):
     # Show incoming changes if in dry-run mode.
     if options.dryrun:
         ui.status("Incoming SVN revisions:")
-        for entry in run_svn_log(repos_url, current_rev, svn_greatest_rev,
+        for entry in run_svn_log(repos_url, next_rev, svn_greatest_rev,
                                  100):
-            msg = entry["message"].splitlines()[0]
-            line = "[%d] %s" % (entry["revision"], msg)
+            if entry["message"]:
+                msg = entry["message"].splitlines()[0].strip()
+            else:
+                msg = ""
+            line = "[%d] %s (%s)" % (entry["revision"], msg, entry["author"])
             # TODO: Line stripping should be done by some simple UI interface.
             if len(line) > 79:
                 line = line[:77]+"..."
@@ -286,7 +296,7 @@ def real_main(options, args):
     except ExternalCommandFailed, e:
         ui.status(str(e), level=ui.ERROR)
     except:
-        ui.status("\nCommand failed with following error:\n", level=ERROR)
+        ui.status("\nCommand failed with following error:\n", level=ui.ERROR)
         traceback.print_exc()
 
 
