@@ -44,6 +44,24 @@ svn_info_xml = """<?xml version="1.0"?>
 </entry>
 </info>"""
 
+svn_info_xml_rev0 = """<?xml version="1.0"?>
+<info>
+<entry
+   kind="dir"
+   path="testsvn"
+   revision="0">
+<url>file:///home/foo/bar</url>
+<repository>
+<root>file:///home/foo/bar</root>
+<uuid>936c9acc-016e-4cc8-a158-a434ae94e2dd</uuid>
+</repository>
+<commit
+   revision="0">
+<date>2009-09-17T19:21:13.394243Z</date>
+</commit>
+</entry>
+</info>"""
+
 class TestSVNInfo(object):
     """
     Tests for handling SVN wc/repo information.
@@ -59,6 +77,15 @@ class TestSVNInfo(object):
         eq_(d['revision'], 20191)
         eq_(d['last_changed_rev'], 20185)
 
+    def test_parse_rev0(self):
+        """
+        Parsing the output of 'svn info --xml' for revision 0.
+        """
+        d = svnclient.parse_svn_info_xml(svn_info_xml_rev0)
+        eq_(d['url'], 'file:///home/foo/bar')
+        eq_(d['repos_url'], 'file:///home/foo/bar')
+        eq_(d['revision'], 0)
+
 
 svn_log_xml = """<?xml version="1.0"?>
 <log>
